@@ -13,8 +13,11 @@
  *                                      Draw() changed for drawing
  *                                      to a pixmap instead of a window.
  *                                      Shorter procedure names.
+ *      klin, Fri Aug 14 15:45:52 1992, patchlevel 4
+ *                                      Minor changes in PS output.
+ *                                      Changed <At/..> to <X11/At/..>.
  */
-static char SCCSid[] = "@(#) Plotter V6.0  92/07/27  AxisCore.c";
+static char SCCSid[] = "@(#) Plotter V6.0  92/08/15  AxisCore.c";
 
 /*
 
@@ -52,13 +55,8 @@ SOFTWARE.
  *   Should not be instansiated, but will sort-of work if required.
  */
 
-#ifdef _AtDevelopment_
-# include "AxisCoreP.h"
-# include "AtConverters.h"
-#else
-# include <At/AxisCoreP.h>
-# include <At/AtConverters.h>
-#endif
+#include <X11/At/AxisCoreP.h>
+#include <X11/At/AtConverters.h>
 
 #define MARGIN      2    /* Pixels between elemnts - will be a resource */
 #define PS_MARGIN   6    /* Same for PS */
@@ -1391,10 +1389,10 @@ int x1, y1, x2, y2, grid_len;
      int tll, max_num_width;
      int x, y;
 
-     fprintf(fp, "%%%% BeginObject: AtAxisCore\n");
+     fprintf(fp, "%%%%BeginObject: AtAxisCore\nGS\n");
 
      /* Draw the line */
-     fprintf(fp, "GS 0.75 setlinewidth [ ] 0 setdash\n");
+     fprintf(fp, "0.75 setlinewidth [ ] 0 setdash\n");
      fprintf(fp, "%d %d M %d %d L\n", x1, y1, x2, y2);
 
      /* Now the frame */
@@ -1571,5 +1569,5 @@ int x1, y1, x2, y2, grid_len;
 	  }
 	  AtTextPSDraw(fp, acp->label_text, x, y);
      }
-     fprintf(fp, "\n%%%% EndObject AtAxisCore\n");
+     fprintf(fp, "\n%%%%EndObject: AtAxisCore\n");
 }
