@@ -11,8 +11,10 @@
  *                                      (neumann@dec4.wu-wien.ac.at)
  *                                      AtTextDraw() and AtTextDrawJustified()
  *                                      changed for drawing to a pixmap
+ *      klin, Sat Aug 15 10:31:50 1992, patchlevel 4
+ *                                      Changed <At/..> to <X11/At/..>.
  */
-static char SCCSid[] = "@(#) Plotter V6.0  92/07/27  Text.c";
+static char SCCSid[] = "@(#) Plotter V6.0  92/08/15  Text.c";
 
 /*
 
@@ -41,11 +43,7 @@ SOFTWARE.
 
 */
 
-#ifdef _AtDevelopment_
-# include "Text.h"
-#else
-# include <At/Text.h>
-#endif
+#include <X11/At/Text.h>
 
 #include <ctype.h>
 
