@@ -12,8 +12,12 @@
  *                                      ClassInitialize().
  *      klin, Sun Aug  2 18:25:28 1992, patchlevel 3
  *                                      Accept min/max values in Initialize().
+ *      klin, Fri Aug  7 10:07:59 1992, Cast type converters to keep
+ *                                      ANSI C compilers quiet
+ *      klin, Sat Aug 15 10:31:50 1992, patchlevel 4
+ *                                      Changed <At/..> to <X11/At/..>.
  */
-static char SCCSid[] = "@(#) Plotter V6.0  92/08/02  XYAxis.c";
+static char SCCSid[] = "@(#) Plotter V6.0  92/08/15  XYAxis.c";
 
 /*
 
@@ -48,11 +52,7 @@ SOFTWARE.
  *   superclass handle all that mucking about with pixels.
  */
 
-#ifdef _AtDevelopment_
-# include "XYAxisP.h"
-#else
-# include <At/XYAxisP.h>
-#endif
+#include <X11/At/XYAxisP.h>
 
 static void ClassInitialize P((void));
 static void Initialize P((AtXYAxisWidget, AtXYAxisWidget));
@@ -832,8 +832,8 @@ void AtRegisterAxisTransformConverter()
      static Boolean registered = False;
 
      if (!registered) {
-	  XtAddConverter(XtRString, XtRTransform, AtCvtStringToAxisTransform,
-			 NULL,0);
+	  XtAddConverter(XtRString, XtRTransform,
+			 (XtConverter) AtCvtStringToAxisTransform, NULL,0);
 	  registered = True;
      }
 }
