@@ -1,4 +1,4 @@
-/* Copyright 1990,91 GROUPE BULL -- See license conditions in file COPYRIGHT */
+/* Copyright 1990-92 GROUPE BULL -- See license conditions in file COPYRIGHT */
 /*****************************************************************************\
 * data.c:                                                                     *
 *                                                                             *
@@ -9,10 +9,10 @@
 \*****************************************************************************/
 
 /* Official version number */
-static char *RCS_Version = "$XpmVersion: 3.2 $";
+static char *RCS_Version = "$XpmVersion: 3.2c $";
 
 /* Internal version number */
-static char *RCS_Id = "$Id: xpm.shar,v 3.8 1992/07/07 08:35:12 lehors Exp $";
+static char *RCS_Id = "$Id: xpm.shar,v 3.13 1992/12/29 16:05:26 lehors Exp $";
 
 #include "xpmP.h"
 #ifdef VMS
@@ -210,18 +210,17 @@ xpmGetString(mdata, sptr, l)
 	    start = mdata->cptr;
 	    while (c = *mdata->cptr)
 		mdata->cptr++;
-	    if (n = mdata->cptr - start) {
-		n++;
-		p = (char *) malloc(n);
-		if (!p)
-		    return (XpmNoMemory);
-		strncpy(p, start, n);
-	    } else
-		free(p);
+	    n = mdata->cptr - start + 1;
+	    p = (char *) malloc(n);
+	    if (!p)
+		return (XpmNoMemory);
+	    strncpy(p, start, n);
 	}
     } else {
 	FILE *file = mdata->stream.file;
 	while (isspace(c = getc(file)) && c != mdata->Eos);
+	if (c == EOF)
+	    return (XpmFileInvalid);
 	p = NULL;
 	i = 0;
 	q = buf;
@@ -247,6 +246,10 @@ xpmGetString(mdata, sptr, l)
 	    i++;
 	    c = getc(file);
 	}
+	if (c == EOF) {
+	    free(p);
+	    return (XpmFileInvalid);
+	}
 	if (n + i != 0) {
 	    /* malloc needed memory */
 	    q = (char *) realloc(p, n + i + 1);
@@ -260,8 +263,10 @@ xpmGetString(mdata, sptr, l)
 	    strncpy(q, buf, i);
 	    n += i;
 	    p[n++] = '\0';
-	} else
-	    free(p);
+	} else {
+	    *p = '\0';
+	    n = 1;
+	}
 	ungetc(c, file);
     }
     *sptr = p;
@@ -381,7 +386,7 @@ xpmWriteFile(filename, mdata)
 /*
  * open the given array to be read or written as an xpmData which is returned
  */
-int
+void
 xpmOpenArray(data, mdata)
     char **data;
     xpmData *mdata;
@@ -393,7 +398,6 @@ xpmOpenArray(data, mdata)
     mdata->CommentLength = 0;
     mdata->Bcmt = mdata->Ecmt = NULL;
     mdata->Bos = mdata->Eos = '\0';
-    return (XpmSuccess);
 }
 
 /*
