@@ -2,31 +2,45 @@
  *      Plot.c
  *
  *      Plotter demo program using motif
- *      klin, Mon Aug  3 16:24:02 1992
+ *      klin, Sat Aug 15 16:08:51 1992
  */
 
 #include <stdio.h>
 #include <math.h>
 #include <Xm/Xm.h>
 
-#include <At/Plotter.h>
-#include <At/XYAxis.h>
-#include <At/XYLinePlot.h>
+#include <X11/At/Plotter.h>
+#include <X11/At/XYAxis.h>
+#include <X11/At/XYLinePlot.h>
 
 #include "mplot.xbm"
 
-static Widget plotter;                  /* The plotter widget */
-static Widget xaxis, yaxis;             /* The axis widgets */
-static Widget line1 = NULL;             /* The 3 demo lines */
-static Widget line2 = NULL;
-static Widget line3 = NULL;
-static Widget line4 = NULL;             /* The 3 user lines */
-static Widget line5 = NULL;
-static Widget line6 = NULL;
+Widget plotter;                         /* The plotter widget */
+Widget xaxis, yaxis;                    /* The axis widgets */
+Widget line1 = NULL;                    /* The 3 demo lines */
+Widget line2 = NULL;
+Widget line3 = NULL;
+Widget line4 = NULL;                    /* The 3 user lines */
+Widget line5 = NULL;
+Widget line6 = NULL;
+
+Widget zplotter = NULL;                 /* The zoom plotter */
+Widget zxaxis, zyaxis;                  /* The zoom axes */
+Widget zline1 = NULL;                   /* The 3 zoom demo lines */
+Widget zline2 = NULL;
+Widget zline3 = NULL;
+Widget zline4 = NULL;                   /* The 3 zoom user lines */
+Widget zline5 = NULL;
+Widget zline6 = NULL;
+
+double zoomxmin, zoomxmax;              /* X values for zooming */
+double zoomymin, zoomymax;              /* Y values for zooming */
+
+static double minxlen, minylen;
 
 #define TITLE  "Plotter Widget Demo"    /* Plotter title */
-#define WIDTH  600                      /* Plotter width */
-#define HEIGHT 400                      /* Plotter height */
+#define WIDTH  640                      /* Plotter width */
+#define HEIGHT 480                      /* Plotter height */
 #define MARGIN 6                        /* Plotter margin */
 #define ZOOM   100.0                    /* Max zoom in % */
 
@@ -45,7 +59,8 @@ August 1992\
 
 #define AUTHOR "\
 \n\
-OSF/Motif Demo Program by Peter Klingebiel (klin@iat.uni-paderborn.de)\n\
+Motif Plotter Demo Program by Peter Klingebiel (klin@iat.uni-paderborn.de)\n\
+with support by the User Interface Builder Motifation\n\
 \n\
 Have fun and enjoy\n"
 
@@ -68,6 +83,11 @@ extern Widget linearx_button, lineary_button;
 extern Widget logarithmicx_button, logarithmicy_button;
 extern Widget subgrid_button;
 extern Widget dialog;
+extern Widget line_button;
+
+static Widget sline = NULL;
+static Widget zsline = NULL;
+static void select_callback();
 
 /*
  *      The plot data
@@ -106,26 +126,31 @@ static void MakePlotter(parent)
   XtSetArg(args[n], XtNtitle, TITLE); n++;
   XtSetArg(args[n], XtNshowTitle, False); n++;
   XtSetArg(args[n], XtNshowLegend, True); n++;
+  XtSetArg(args[n], XtNusePixmap, True); n++;
   plotter = XtCreateManagedWidget("plotter", atPlotterWidgetClass, parent, args, n);
+  XtAddCallback(plotter, XtNselectCallback, select_callback, NULL);
 
   /* Create the X axis */
   n = 0;
   XtSetArg(args[n], XtNlabel, "Time"); n++;
   XtSetArg(args[n], XtNaxisWidth, 1); n++;
-  xaxis = XtCreateWidget("xaxis", atXYAxisWidgetClass, plotter, args, n);
+  xaxis = XtCreateWidget("axis", atXYAxisWidgetClass, plotter, args, n);
 
   /* Create the Y axis */
   n = 0;
   XtSetArg(args[n], XtNlabel, "Amplitude"); n++;
   XtSetArg(args[n], XtNaxisWidth, 1); n++;
   XtSetArg(args[n], XtNvertical, True); n++;
-  yaxis = XtCreateWidget("yaxis", atXYAxisWidgetClass, plotter, args, n);
+  yaxis = XtCreateWidget("axis", atXYAxisWidgetClass, plotter, args, n);
 
   /* Attach X and Y axes to the plotter */
   n = 0;
   XtSetArg(args[n], XtNxAxis, xaxis); n++;
   XtSetArg(args[n], XtNyAxis, yaxis); n++;
   XtSetValues(plotter, args, n);
+
+  /* Create the zoom dialog and plotter */
+  MakeZoomDialogAndPlotter();
 }
 
 /*
@@ -141,7 +166,8 @@ void MakeDemoLines()
   XtSetArg(args[n], XtNlegendName, "sinus-points"); n++;
   XtSetArg(args[n], XtNplotLineStyle, AtLineDOTTED); n++;
   XtSetArg(args[n], XtNplotMarkType,  AtMarkRECTANGLE); n++;
-  line1 = XtCreateWidget("line1", atXYLinePlotWidgetClass, plotter, args, n);
+  line1 = XtCreateWidget("line1",  atXYLinePlotWidgetClass, plotter, args, n);
+  zline1 = XtCreateWidget("line1", atXYLinePlotWidgetClass, zplotter, args, n);
 
   /* Create line 2 */
   n = 0;
@@ -149,12 +175,14 @@ void MakeDemoLines()
   XtSetArg(args[n], XtNplotLineStyle, AtLineDOTDASHED); n++;
   XtSetArg(args[n], XtNplotMarkType,  AtMarkXMARK); n++;
   line2 = XtCreateWidget("line2", atXYLinePlotWidgetClass, plotter, args, n);
+  zline2 = XtCreateWidget("line2", atXYLinePlotWidgetClass, zplotter, args, n);
 
   /* Create line 3 */
   n = 0;
   XtSetArg(args[n], XtNlegendName, "polygon"); n++;
   XtSetArg(args[n], XtNplotMarkType,  AtMarkSTAR); n++;
   line3 = XtCreateWidget("line3", atXYLinePlotWidgetClass, plotter, args, n);
+  zline3 = XtCreateWidget("line3", atXYLinePlotWidgetClass, zplotter, args, n);
 }
 
 /*
@@ -197,6 +225,19 @@ static void AttachData()
 
   /* Attach polygon from predefined type AtDoublePoint to line3 */
   AtXYLinePlotAttachDoublePoints(line3, &polygon[0], 1, PNUM);
+
+  /* Attach curve from own defined type point to zline1 and zline 2*/
+  AtXYLinePlotAttachData(zline1,
+    (XtPointer) &curve[0].xval,  AtDouble, sizeof(struct point),
+    (XtPointer) &curve[0].yval1, AtDouble, sizeof(struct point),
+    1, CNUM);
+  AtXYLinePlotAttachData(zline2,
+    (XtPointer) &curve[0].xval,  AtDouble, sizeof(struct point),
+    (XtPointer) &curve[0].yval2, AtDouble, sizeof(struct point),
+    1, CNUM);
+
+  /* Attach polygon from predefined type AtDoublePoint to zline3 */
+  AtXYLinePlotAttachDoublePoints(zline3, &polygon[0], 1, PNUM);
 }
 
 /*
@@ -233,6 +274,8 @@ static char *ReadPlotData(filename)
 	XtSetArg(args[n], XtNplotLineStyle, AtLineSOLID), n++;
 	line4 = XtCreateWidget("line4", atXYLinePlotWidgetClass, plotter, args, n);
 	AtXYLinePlotAttachDoublePoints(line4, data, 1, nd);
+	zline4 = XtCreateWidget("line4", atXYLinePlotWidgetClass, zplotter, args, n);
+	AtXYLinePlotAttachDoublePoints(zline4, data, 1, nd);
       }
       else if(line5 == NULL) {
 	n = 0;
@@ -242,6 +285,8 @@ static char *ReadPlotData(filename)
 	XtSetArg(args[n], XtNplotLineStyle, AtLineDASHED), n++;
 	line5 = XtCreateWidget("line5", atXYLinePlotWidgetClass, plotter, args, n);
 	AtXYLinePlotAttachDoublePoints(line5, data, 1, nd);
+	zline5 = XtCreateWidget("line5", atXYLinePlotWidgetClass, zplotter, args, n);
+	AtXYLinePlotAttachDoublePoints(zline5, data, 1, nd);
       }
       else if(line6 == NULL) {
 	n = 0;
@@ -251,6 +296,8 @@ static char *ReadPlotData(filename)
 	XtSetArg(args[n], XtNplotLineStyle, AtLineSOLID), n++;
 	line6 = XtCreateWidget("line6", atXYLinePlotWidgetClass, plotter, args, n);
 	AtXYLinePlotAttachDoublePoints(line6, data, 1, nd);
+	zline6 = XtCreateWidget("line6", atXYLinePlotWidgetClass, zplotter, args, n);
+	AtXYLinePlotAttachDoublePoints(zline6, data, 1, nd);
       }
       errmsg = NULL;
     }
@@ -267,11 +314,55 @@ static char *ReadPlotData(filename)
   return(errmsg);
 }
 
+/*
+ *      The line plot select callback
+ */
+
+static void select_callback(widget, client_data, call_data)
+  Widget widget;
+  XtPointer client_data;
+  AtSelectCallbackData *call_data;
+{
+  Widget w;
+
+  sline = zsline = NULL;
+  if(call_data->reason == AtSelectSELECTED) {
+    w = call_data->widget;
+    if(w == (Widget) line1) {
+      sline  = line1;
+      zsline = zline1;
+    }
+    else if(w == (Widget) line2) {
+      sline  = line2;
+      zsline = zline2;
+    }
+    else if(w == (Widget) line3) {
+      sline  = line3;
+      zsline = zline3;
+    }
+    else if(w == (Widget) line4) {
+      sline  = line4;
+      zsline = zline4;
+    }
+    else if(w == (Widget) line5) {
+      sline  = line5;
+      zsline = zline5;
+    }
+    else if(w == (Widget) line6) {
+      sline  = line6;
+      zsline = zline6;
+    }
+  }
+  XtSetSensitive(line_button, sline ? True : False);
+}
+
 /*
  *      The mainmenu callbacks
  */
 
+#ifndef __STDC__
 extern int exit();
+#endif
 
 /* callback from dialog */
 static char *plotfilename = NULL;
@@ -382,14 +473,12 @@ void askposition_callback()
     XtSetArg(args[0], XmNvalue, "");
     XtSetValues(x_text, args, 1);
     XtSetValues(y_text, args, 1);
-    XtSetArg(args[0], XmNsensitive, False);
-    XtSetValues(ask_label, args, 1);
+    XtSetSensitive(ask_label, False);
     is_asking = False;
   }
   else {
     XtAddCallback(plotter, XtNclickCallback, get_click_callback, NULL);
-    XtSetArg(args[0], XmNsensitive, True);
-    XtSetValues(ask_label, args, 1);
+    XtSetSensitive(ask_label, True);
     is_asking = True;
   }
 }
@@ -419,19 +508,17 @@ void getposition_callback()
 
   if(is_positioning) {
     XtRemoveAllCallbacks(plotter, XtNmotionCallback);
-    XtSetArg(args[0], XmNsensitive, False);
-    XtSetValues(xpos_label, args, 1);
-    XtSetValues(ypos_label, args, 1);
+    XtSetSensitive(xpos_label, False);
+    XtSetSensitive(ypos_label, False);
     XtSetArg(args[0], XmNvalue, "");
     XtSetValues(x_text, args, 1);
     XtSetValues(y_text, args, 1);
     is_positioning = False;
   }
   else {
-    XtSetArg(args[0], XmNsensitive, True);
-    XtSetValues(xpos_label, args, 1);
-    XtSetValues(ypos_label, args, 1);
     XtAddCallback(plotter, XtNmotionCallback, get_motion_callback, NULL);
+    XtSetSensitive(xpos_label, True);
+    XtSetSensitive(ypos_label, True);
     is_positioning = True;
   }
 }
@@ -443,27 +530,16 @@ void plotterform_callback(widget, client_data, event, cont)
   Boolean cont;
 {
   Arg args[2];
-  Boolean enter = (Boolean) client_data;
 
-  if(enter) {
-    XtSetArg(args[0], XmNshadowType, XmSHADOW_IN);
+  if(is_positioning && !is_asking) {
+    XtSetArg(args[0], XmNvalue, "");
+    XtSetValues(x_text, args, 1);
+    XtSetValues(y_text, args, 1);
   }
-  else {
-    if(is_positioning && !is_asking) {
-      XtSetArg(args[0], XmNvalue, "");
-      XtSetValues(x_text, args, 1);
-      XtSetValues(y_text, args, 1);
-    }
-    XtSetArg(args[0], XmNshadowType, XmSHADOW_OUT);
-  }
-  XtSetValues(plotter_frame, args, 1);
 }
 
 /* plot_pulldown: zoom_button */
 
-static double oldxmin, oldxmax, minxlen;
-static double oldymin, oldymax, minylen;
-
 void get_drag_callback(widget, client_data, call_data)
   Widget widget;
   XtPointer client_data;
@@ -471,7 +547,7 @@ void get_drag_callback(widget, client_data, call_data)
 {
   Arg args[4];
   char msg[256];
-  static double x1, y1, x2, y2;
+  double x1, y1, x2, y2;
   double xl, yl;
 
   x1 = call_data->x11;
@@ -488,7 +564,7 @@ void get_drag_callback(widget, client_data, call_data)
     XtSetArg(args[0], XtNmin, &x1);
     XtSetArg(args[1], XtNmax, &x2);
     XtSetArg(args[2], XtNautoScale, False);
-    XtSetValues(xaxis, args, 3);
+    XtSetValues(zxaxis, args, 3);
   }
   if(yl < minylen) {
     sprintf(msg, "Warning: Zooming not possible for Y axis (>%.0lf%%)", ZOOM);
@@ -498,42 +574,33 @@ void get_drag_callback(widget, client_data, call_data)
     XtSetArg(args[0], XtNmin, &y1);
     XtSetArg(args[1], XtNmax, &y2);
     XtSetArg(args[2], XtNautoScale, False);
-    XtSetValues(yaxis, args, 3);
+    XtSetValues(zyaxis, args, 3);
   }
+  Popup_zoom(x1, x2, y1, y2);
 }
 
 void zoom_callback()
 {
-  Arg args[6];
-  int n;
   static Boolean is_zooming = False;
 
   if(is_zooming) {
     XtRemoveAllCallbacks(plotter, XtNdragCallback);
-    XtSetArg(args[0], XtNmin, &oldxmin);
-    XtSetArg(args[1], XtNmax, &oldxmax);
-    XtSetArg(args[2], XtNautoScale, True);
-    XtSetValues(xaxis, args, 3);
-    XtSetArg(args[0], XtNmin, &oldymin);
-    XtSetArg(args[1], XtNmax, &oldymax);
-    XtSetArg(args[2], XtNautoScale, True);
-    XtSetValues(yaxis, args, 3);
-    XtSetArg(args[0], XmNsensitive, False);
-    XtSetValues(zoom_label, args, 1);
+    XtSetSensitive(zoom_label, False);
+    Popdown_zoom();
     is_zooming = False;
   }
   else {
-    XtVaGetValues(xaxis, XtNmin, &oldxmin, XtNmax, &oldxmax, NULL);
-    XtVaGetValues(yaxis, XtNmin, &oldymin, XtNmax, &oldymax, NULL);
-    minxlen = (oldxmax - oldxmin) / ZOOM;
-    minylen = (oldymax - oldymin) / ZOOM;
+    XtVaGetValues(xaxis, XtNmin, &zoomxmin, XtNmax, &zoomxmax, NULL);
+    XtVaGetValues(yaxis, XtNmin, &zoomymin, XtNmax, &zoomymax, NULL);
+    minxlen = (zoomxmax - zoomxmin) / ZOOM;
+    minylen = (zoomymax - zoomymin) / ZOOM;
     XtAddCallback(plotter, XtNdragCallback, get_drag_callback, NULL);
-    XtSetArg(args[0], XmNsensitive, True);
-    XtSetValues(zoom_label, args, 1);
+    XtSetSensitive(zoom_label, True);
     is_zooming = True;
   }
 }
 
+
 /* plot_pulldown: legend_button */
 void legend_callback()
 {
@@ -613,6 +680,7 @@ void linearx_callback()
 
   XtSetArg(args[0], XtNaxisTransform, AtTransformLINEAR);
   XtSetValues(xaxis, args, 1);
+  XtSetValues(zxaxis, args, 1);
 }
 
 /* axis_pulldown/transform_pulldown: logarithmicx_button */
@@ -622,6 +690,7 @@ void logarithmicx_callback()
 
   XtSetArg(args[0], XtNaxisTransform, AtTransformLOGARITHMIC);
   XtSetValues(xaxis, args, 1);
+  XtSetValues(zxaxis, args, 1);
 }
 /* axis_pulldown/transform_pulldown: lineary_button */
 void lineary_callback()
@@ -630,6 +699,7 @@ void lineary_callback()
 
   XtSetArg(args[0], XtNaxisTransform, AtTransformLINEAR);
   XtSetValues(yaxis, args, 1);
+  XtSetValues(zyaxis, args, 1);
 }
 
 /* axis_pulldown/transform_pulldown: logarithmicy_button */
@@ -639,6 +709,7 @@ void logarithmicy_callback()
 
   XtSetArg(args[0], XtNaxisTransform, AtTransformLOGARITHMIC);
   XtSetValues(yaxis, args, 1);
+  XtSetValues(zyaxis, args, 1);
 }
 
 /* axis_pulldown/round button: roundx_button */
@@ -682,6 +753,8 @@ void grid_callback()
   }
   XtSetValues(xaxis, aargs, 1);
   XtSetValues(yaxis, aargs, 1);
+  XtSetValues(zxaxis, aargs, 1);
+  XtSetValues(zyaxis, aargs, 1);
   XtSetValues(subgrid_button, gargs, 1);
 }
 
@@ -697,6 +770,8 @@ void subgrid_callback()
   XtSetArg(args[0], XtNdrawSubgrid, gn);
   XtSetValues(xaxis, args, 1);
   XtSetValues(yaxis, args, 1);
+  XtSetValues(zxaxis, args, 1);
+  XtSetValues(zyaxis, args, 1);
 }
 
 /* line_pulldown: linewidth_buttons 0..3 contained in client_data */
@@ -708,18 +783,8 @@ void width_callback(widget, client_data, call_data)
   Arg args[2];
 
   XtSetArg(args[0], XtNlineWidth, client_data);
-  if(line1)
-    XtSetValues(line1, args, 1);
-  if(line2)
-    XtSetValues(line2, args, 1);
-  if(line3)
-    XtSetValues(line3, args, 1);
-  if(line4)
-    XtSetValues(line4, args, 1);
-  if(line5)
-    XtSetValues(line5, args, 1);
-  if(line6)
-    XtSetValues(line6, args, 1);
+  XtSetValues(sline,  args, 1);
+  XtSetValues(zsline, args, 1);
 }
 
 /* line_pulldown: linetype_buttons. Type is contained in client_data */
@@ -742,18 +807,8 @@ void type_callback(widget, client_data, call_data)
     default: return;
   }
   XtSetArg(args[0], XtNplotLineType, type);
-  if(line1)
-    XtSetValues(line1, args, 1);
-  if(line2)
-    XtSetValues(line2, args, 1);
-  if(line3)
-    XtSetValues(line3, args, 1);
-  if(line4)
-    XtSetValues(line4, args, 1);
-  if(line5)
-    XtSetValues(line5, args, 1);
-  if(line6)
-    XtSetValues(line6, args, 1);
+  XtSetValues(sline,  args, 1);
+  XtSetValues(zsline, args, 1);
 }
 
 /* line_pulldown: linestyle_buttons. Style is contained in client_data */
@@ -776,18 +831,46 @@ void style_callback(widget, client_data, call_data)
     default: return;
   }
   XtSetArg(args[0], XtNplotLineStyle, style);
-  if(line1)
-    XtSetValues(line1, args, 1);
-  if(line2)
-    XtSetValues(line2, args, 1);
-  if(line3)
-    XtSetValues(line3, args, 1);
-  if(line4)
-    XtSetValues(line4, args, 1);
-  if(line5)
-    XtSetValues(line5, args, 1);
-  if(line6)
-    XtSetValues(line6, args, 1);
+  XtSetValues(sline,  args, 1);
+  XtSetValues(zsline, args, 1);
+}
+
+/* line_pulldown: xoffset_buttons */
+void xoffset_callback(widget, client_data, call_data)
+  Widget widget;
+  XtPointer client_data;
+  XmAnyCallbackStruct *call_data;
+{
+  static double off = 0.0;
+  Arg args[2];
+  Boolean plus = (Boolean) client_data;
+
+  if(plus)
+    off += 0.125;
+  else
+    off -= 0.125;
+  XtSetArg(args[0], XtNxOffset, &off);
+  XtSetValues(sline, args, 1);
+  XtSetValues(zsline, args, 1);
+}
+
+/* line_pulldown: yoffset_buttons */
+void yoffset_callback(widget, client_data, call_data)
+  Widget widget;
+  XtPointer client_data;
+  XmAnyCallbackStruct *call_data;
+{
+  static double off = 0.0;
+  Arg args[2];
+  Boolean plus = (Boolean) client_data;
+
+  if(plus)
+    off += 0.125;
+  else
+    off -= 0.125;
+  XtSetArg(args[0], XtNyOffset, &off);
+  XtSetValues(sline, args, 1);
+  XtSetValues(zsline, args, 1);
 }
 
 /* help_pulldown: version_button */
