@@ -1,4 +1,4 @@
-/* Copyright 1990,91 GROUPE BULL -- See license conditions in file COPYRIGHT */
+/* Copyright 1990-92 GROUPE BULL -- See license conditions in file COPYRIGHT */
 /*****************************************************************************\
 * parse.c:                                                                    *
 *                                                                             *
@@ -93,8 +93,11 @@ xpmParseData(data, attrib_return, attributes)
     /*
      * init the hastable
      */
-    if (USE_HASHTABLE)
-	xpmHashTableInit(&hashtable);
+    if (USE_HASHTABLE) {
+	ErrorStatus = xpmHashTableInit(&hashtable);
+	if (ErrorStatus != XpmSuccess)
+	    return(ErrorStatus);
+    }
     
     /*
      * read colors 
@@ -133,13 +136,16 @@ xpmParseData(data, attrib_return, attributes)
     /*
      * parse extensions
      */
-    if (extensions && attributes
-	&& (attributes->valuemask & XpmReturnExtensions)) {
-	ErrorStatus = ParseExtensions(data, &attributes->extensions,
-				      &attributes->nextensions);
-	if (ErrorStatus != XpmSuccess)
-	    RETURN(ErrorStatus);
-    }
+    if (attributes && (attributes->valuemask & XpmReturnExtensions)) 
+	if (extensions) {
+	    ErrorStatus = ParseExtensions(data, &attributes->extensions,
+					  &attributes->nextensions);
+	    if (ErrorStatus != XpmSuccess)
+		RETURN(ErrorStatus);
+	} else {
+	    attributes->extensions = NULL;
+	    attributes->nextensions = 0;
+	}
 
     /*
      * store found informations in the xpmInternAttrib structure 
@@ -217,6 +223,7 @@ ParseColors(data, ncolors, cpp, colorTablePtr, hashtable)
     char curbuf[BUFSIZ];		/* current buffer */
     char ***ct, **cts, **sptr, *s;
     char ***colorTable;
+    int ErrorStatus;
 
     colorTable = (char ***) calloc(ncolors, sizeof(char **));
     if (!colorTable)
@@ -243,10 +250,15 @@ ParseColors(data, ncolors, cpp, colorTablePtr, hashtable)
 	*s = '\0';
 
 	/*
-	 * tore the string in the hashtable with its color index number
+	 * store the string in the hashtable with its color index number
 	 */
-	if (USE_HASHTABLE)
-	    xpmHashIntern(hashtable, *cts, HashAtomData(a));
+	if (USE_HASHTABLE) {
+	    ErrorStatus = xpmHashIntern(hashtable, *cts, HashAtomData(a));
+	    if (ErrorStatus != XpmSuccess) {
+		xpmFreeColorTable(colorTable, ncolors);
+		return(ErrorStatus);
+	    }
+	}
 
 	/*
 	 * read color keys and values 
@@ -310,49 +322,109 @@ ParsePixels(data, width, height, ncolors, cpp, colorTable, hashtable, pixels)
     unsigned int **pixels;
 {
     unsigned int *iptr, *iptr2;
-    char *s;
     unsigned int a, x, y;
-    char buf[BUFSIZ];
 
     iptr2 = (unsigned int *) malloc(sizeof(unsigned int) * width * height);
     if (!iptr2)
 	return(XpmNoMemory);
 
-    buf[cpp] = '\0';
     iptr = iptr2;
-    if (USE_HASHTABLE) {
-	xpmHashAtom *slot;
 
-	for (y = 0; y < height; y++) {
-	    xpmNextString(data);
-	    for (x = 0; x < width; x++, iptr++) {
-		for (a = 0, s = buf; a < cpp; a++, s++)
-		    *s = xpmGetC(data);
-		slot = xpmHashSlot(hashtable, buf);
-		if (!*slot) {		/* no color matches */
-		    free(iptr2);
-		    return(XpmFileInvalid);
-		}
-		*iptr = HashColorIndex(slot);
-	    }
-	}
-    } else {
-	for (y = 0; y < height; y++) {
-	    xpmNextString(data);
-	    for (x = 0; x < width; x++, iptr++) {
-		for (a = 0, s = buf; a < cpp; a++, s++)
-		    *s = xpmGetC(data);
-		for (a = 0; a < ncolors; a++)
-		    if (!strcmp(colorTable[a][0], buf))
-			break;
-		if (a == ncolors) {	/* no color matches */
-		    free(iptr2);
-		    return(XpmFileInvalid);
-		}
-		*iptr = a;
-	    }
-	}
-    }
+    switch (cpp) {
+
+       case (1): /* Optimize for single character colors */
+       {
+	   unsigned short colidx[256];
+
+	   bzero(colidx, 256 * sizeof(short));
+	   for (a = 0; a < ncolors; a++)
+	       colidx[ colorTable[a][0][0] ] = a + 1;
+                
+	   for (y = 0; y < height; y++) 
+	   {
+	       xpmNextString(data);
+	       for (x = 0; x < width; x++, iptr++)
+	       {
+		   int idx = colidx[xpmGetC(data)];
+		   if ( idx != 0 )
+		       *iptr = idx - 1;
+		   else {
+		       free(iptr2);
+		       return(XpmFileInvalid);
+		   }
+	       }
+	   }
+       }
+       break;
+
+       case (2): /* Optimize for double character colors */
+       {
+	   unsigned short cidx[256][256];
+
+	   bzero(cidx, 256*256 * sizeof(short));
+	   for (a = 0; a < ncolors; a++)
+	       cidx [ colorTable[a][0][0] ][ colorTable[a][0][1] ] = a + 1;
+
+	   for (y = 0; y < height; y++) 
+	   {
+	       xpmNextString(data);
+	       for (x = 0; x < width; x++, iptr++)
+	       {
+		   int cc1 = xpmGetC(data);
+		   int idx = cidx[cc1][ xpmGetC(data) ];
+		   if ( idx != 0 )
+		       *iptr = idx - 1;
+		   else {
+		       free(iptr2);
+		       return(XpmFileInvalid);
+		   }
+	       }
+	   }
+       }
+       break;
+
+       default : /* Non-optimized case of long color names */
+       {
+	   char *s;
+	   char buf[BUFSIZ];
+
+	   buf[cpp] = '\0';
+	   if (USE_HASHTABLE) {
+	       xpmHashAtom *slot;
+
+	       for (y = 0; y < height; y++) {
+		   xpmNextString(data);
+		   for (x = 0; x < width; x++, iptr++) {
+		       for (a = 0, s = buf; a < cpp; a++, s++)
+			   *s = xpmGetC(data);
+		       slot = xpmHashSlot(hashtable, buf);
+		       if (!*slot) {		/* no color matches */
+			   free(iptr2);
+			   return(XpmFileInvalid);
+		       }
+		       *iptr = HashColorIndex(slot);
+		   }
+	       }
+	   } else {
+	       for (y = 0; y < height; y++) {
+		   xpmNextString(data);
+		   for (x = 0; x < width; x++, iptr++) {
+		       for (a = 0, s = buf; a < cpp; a++, s++)
+			   *s = xpmGetC(data);
+		       for (a = 0; a < ncolors; a++)
+			   if (!strcmp(colorTable[a][0], buf))
+			       break;
+		       if (a == ncolors) {	/* no color matches */
+			   free(iptr2);
+			   return(XpmFileInvalid);
+		       }
+		       *iptr = a;
+		   }
+	       }
+	   }
+       }
+       break;
+   }
     *pixels = iptr2;
     return (XpmSuccess);
 }
@@ -365,7 +437,7 @@ ParseExtensions(data, extensions, nextensions)
 {
     XpmExtension *exts = NULL, *ext;
     unsigned int num = 0;
-    unsigned int nlines, a, l, notstart, notend;
+    unsigned int nlines, a, l, notstart, notend = 0;
     int status;
     char *string, *s, *s2, **sp;
 
@@ -388,11 +460,14 @@ ParseExtensions(data, extensions, nextensions)
 	    return(status);
 	}
     }
+    if (!notstart)
+	notend = strncmp("XPMENDEXT", string, 9);
     while (!notstart && notend) {
 	/* there starts an extension */
 	ext = (XpmExtension *) realloc(exts, (num + 1) * sizeof(XpmExtension));
 	if (!ext) {
-	    free(exts);
+	    free(string);
+	    XpmFreeExtensions(exts, num);
 	    return(XpmNoMemory);
 	}
 	exts = ext;
@@ -404,7 +479,10 @@ ParseExtensions(data, extensions, nextensions)
 	a = s2 - s;
 	ext->name = (char *) malloc(l - a - 6);
 	if (!ext->name) {
-	    free(exts);
+	    free(string);
+	    ext->lines = NULL;
+	    ext->nlines = 0;
+	    XpmFreeExtensions(exts, num + 1);
 	    return(XpmNoMemory);
 	}
 	strncpy(ext->name, s + a, l - a - 6);
@@ -413,7 +491,9 @@ ParseExtensions(data, extensions, nextensions)
 	xpmNextString(data);
 	status = xpmGetString(data, &string, &l);
 	if (status != XpmSuccess) {
-	    free(exts);
+	    ext->lines = NULL;
+	    ext->nlines = 0;
+	    XpmFreeExtensions(exts, num + 1);
 	    return(status);
 	}
 	ext->lines = (char **) malloc(sizeof(char *));
@@ -422,7 +502,9 @@ ParseExtensions(data, extensions, nextensions)
 	       && (notend = strncmp("XPMENDEXT", string, 9))) {
 	    sp = (char **) realloc(ext->lines, (nlines + 1) * sizeof(char *));
 	    if (!sp) {
-		free(exts);
+		free(string);
+		ext->nlines = nlines;
+		XpmFreeExtensions(exts, num + 1);
 		return(XpmNoMemory);
 	    }
 	    ext->lines = sp;
@@ -431,7 +513,8 @@ ParseExtensions(data, extensions, nextensions)
 	    xpmNextString(data);
 	    status = xpmGetString(data, &string, &l);
 	    if (status != XpmSuccess) {
-		free(exts);
+		ext->nlines = nlines;
+		XpmFreeExtensions(exts, num + 1);
 		return(status);
 	    }
 	}
@@ -446,8 +529,7 @@ ParseExtensions(data, extensions, nextensions)
 	free(string);
 	free(exts);
 	exts = NULL;
-    }
-    if (!notend)
+    } else if (!notend)
 	free(string);
     *nextensions = num;
     *extensions = exts;
