@@ -16,7 +16,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /user6/ouster/tcl/RCS/tclCmdAH.c,v 1.73 91/11/07 09:02:11 ouster Exp $ SPRITE (Berkeley)";
+static char rcsid[] = "$Header: /user6/ouster/tcl/RCS/tclCmdAH.c,v 1.78 92/08/25 20:47:36 ouster Exp $ SPRITE (Berkeley)";
 #endif
 
 #include "tclInt.h"
@@ -175,7 +175,7 @@ Tcl_CaseCmd(dummy, interp, argc, argv)
 	result = Tcl_Eval(interp, caseArgv[body], 0, (char **) NULL);
 	if (result == TCL_ERROR) {
 	    char msg[100];
-	    sprintf(msg, "\n    (\"%.50s\" arm line %d)", caseArgv[i],
+	    sprintf(msg, "\n    (\"%.50s\" arm line %d)", caseArgv[body-1],
 		    interp->errorLine);
 	    Tcl_AddErrorInfo(interp, msg);
 	}
@@ -647,6 +647,7 @@ Tcl_FormatCmd(dummy, interp, argc, argv)
     int noPercent;		/* Special case for speed:  indicates there's
 				 * no field specifier, just a string to copy. */
     char **curArg;		/* Remainder of argv array. */
+    int useShort;		/* Value to be printed is short (half word). */
 
     /*
      * This procedure is a bit nasty.  The goal is to use sprintf to
@@ -671,7 +672,7 @@ Tcl_FormatCmd(dummy, interp, argc, argv)
     for (format = argv[1]; *format != 0; ) {
 	register char *newPtr = newFormat;
 
-	width = precision = useTwoWords = noPercent = 0;
+	width = precision = useTwoWords = noPercent = useShort = 0;
 
 	/*
 	 * Get rid of any characters before the next field specifier.
@@ -718,16 +719,12 @@ Tcl_FormatCmd(dummy, interp, argc, argv)
 	*newPtr = '%';
 	newPtr++;
 	format++;
-	while ((*format == '-') || (*format == '#')) {
+	while ((*format == '-') || (*format == '#') || (*format == '0')
+		|| (*format == ' ') || (*format == '+')) {
 	    *newPtr = *format;
 	    newPtr++;
 	    format++;
 	}
-	if (*format == '0') {
-	    *newPtr = '0';
-	    newPtr++;
-	    format++;
-	}
 	if (isdigit(*format)) {
 	    width = atoi(format);
 	    do {
@@ -779,6 +776,11 @@ Tcl_FormatCmd(dummy, interp, argc, argv)
 	}
 	if (*format == 'l') {
 	    format++;
+	} else if (*format == 'h') {
+	    useShort = 1;
+	    *newPtr = 'h';
+	    newPtr++;
+	    format++;
 	}
 	*newPtr = *format;
 	newPtr++;
@@ -790,9 +792,13 @@ Tcl_FormatCmd(dummy, interp, argc, argv)
 	    case 'D':
 	    case 'O':
 	    case 'U':
-		*newPtr = tolower(*format);
-		newPtr[-1] = 'l';
-		newPtr++;
+		if (!useShort) {
+		    newPtr++;
+		} else {
+		    useShort = 0;
+		}
+		newPtr[-1] = tolower(*format);
+		newPtr[-2] = 'l';
 		*newPtr = 0;
 	    case 'd':
 	    case 'o':
@@ -869,14 +875,16 @@ Tcl_FormatCmd(dummy, interp, argc, argv)
 	    dstSpace = newSpace;
 	}
 	if (noPercent) {
-	    memcpy((VOID *) dst+dstSize, (VOID *) oneWordValue, size);
+	    memcpy((VOID *) (dst+dstSize), (VOID *) oneWordValue, size);
 	    dstSize += size;
 	    dst[dstSize] = 0;
 	} else {
 	    if (useTwoWords) {
 		sprintf(dst+dstSize, newFormat, twoWordValue);
+	    } else if (useShort) {
+		sprintf(dst+dstSize, newFormat, (short) oneWordValue);
 	    } else {
-		sprintf(dst+dstSize, newFormat, oneWordValue);
+		sprintf(dst+dstSize, newFormat, (char *) oneWordValue);
 	    }
 	    dstSize += strlen(dst+dstSize);
 	}
