@@ -16,7 +16,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /user6/ouster/tcl/RCS/tclParse.c,v 1.21 92/06/08 09:32:37 ouster Exp $ SPRITE (Berkeley)";
+static char rcsid[] = "$Header: /user6/ouster/tcl/RCS/tclParse.c,v 1.24 92/12/09 16:54:48 ouster Exp $ SPRITE (Berkeley)";
 #endif
 
 #include "tclInt.h"
@@ -176,7 +176,7 @@ Tcl_Backslash(src, readPtr)
 		    break;
 		}
 		count = 4;
-		result = (*p & 037) | 0200;
+		result = (*p & 037) | '\200';
 		break;
 	    }
 	    count = 3;
@@ -190,7 +190,7 @@ Tcl_Backslash(src, readPtr)
 		break;
 	    }
 	    count = 3;
-	    result = *p + 0200;
+	    result = *p + '\200';
 	    break;
 	case '}':
 	case '{':
@@ -861,10 +861,10 @@ TclExpandParseValue(pvPtr, needed)
  *	word of the command.
  *
  * Results:
- *	The return value is a pointer to the character just after the
- *	last one that's part of the word pointed to by "start".  This
- *	may be the address of the NULL character at the end of the
- *	string.
+ *	The return value is a pointer to the last character that's part
+ *	of the word pointed to by "start".  If the word doesn't end
+ *	properly within the string then the return value is the address
+ *	of the null character at the end of the string.
  *
  * Side effects:
  *	None.
@@ -893,6 +893,10 @@ TclWordEnd(start, nested)
 
     if (*p == '"') {
 	p = QuoteEnd(p+1, '"');
+	if (*p == 0) {
+	    return p;
+	}
+	p++;
     } else if (*p == '{') {
 	int braces = 1;
 	while (braces != 0) {
@@ -909,6 +913,7 @@ TclWordEnd(start, nested)
 		return p;
 	    }
 	}
+	p++;
     }
 
     /*
@@ -919,39 +924,47 @@ TclWordEnd(start, nested)
      * for here the garbage is treated as part of the word.
      */
 
-    while (*p != 0) {
+    while (1) {
 	if (*p == '[') {
-	    p++;
-	    while ((*p != ']') && (*p != 0)) {
+	    for (p++; *p != ']'; p++) {
 		p = TclWordEnd(p, 1);
+		if (*p == 0) {
+		    return p;
+		}
 	    }
-	    if (*p == ']') {
-		p++;
-	    }
+	    p++;
 	} else if (*p == '\\') {
 	    (void) Tcl_Backslash(p, &count);
 	    p += count;
 	} else if (*p == '$') {
 	    p = VarNameEnd(p);
+	    if (*p == 0) {
+		return p;
+	    }
+	    p++;
 	} else if (*p == ';') {
 	    /*
-	     * Note:  semi-colon terminates a word
-	     * and also counts as a word by itself.
+	     * Include the semi-colon in the word that is returned.
 	     */
 
-	    if (p == start) {
-		p++;
-	    }
-	    break;
+	    return p;
 	} else if (isspace(*p)) {
-	    break;
+	    return p-1;
 	} else if ((*p == ']') && nested) {
-	    break;
+	    return p-1;
+	} else if (*p == 0) {
+	    if (nested) {
+		/*
+		 * Nested commands can't end because of the end of the
+		 * string.
+		 */
+		return p;
+	    }
+	    return p-1;
 	} else {
 	    p++;
 	}
     }
-    return p;
 }
 
 /*
@@ -965,8 +978,11 @@ TclWordEnd(start, nested)
  *	index name.
  *
  * Results:
- *	The return value is a pointer to the character just after the
- *	last one that is part of the quoted string.
+ *	The return value is a pointer to the last character that is
+ *	part of the quoted string (i.e the character that's equal to
+ *	term).  If the quoted string doesn't terminate properly then
+ *	the return value is a pointer to the null character at the
+ *	end of the string.
  *
  * Side effects:
  *	None.
@@ -984,25 +1000,31 @@ QuoteEnd(string, term)
     register char *p = string;
     int count;
 
-    while ((*p != 0) && (*p != term)) {
+    while (*p != term) {
 	if (*p == '\\') {
 	    (void) Tcl_Backslash(p, &count);
 	    p += count;
 	} else if (*p == '[') {
-	    p++;
-	    while ((*p != ']') && (*p != 0)) {
+	    for (p++; *p != ']'; p++) {
 		p = TclWordEnd(p, 1);
+		if (*p == 0) {
+		    return p;
+		}
 	    }
-	    if (*p == ']') {
-		p++;
-	    }
+	    p++;
 	} else if (*p == '$') {
 	    p = VarNameEnd(p);
+	    if (*p == 0) {
+		return p;
+	    }
+	    p++;
+	} else if (*p == 0) {
+	    return p;
 	} else {
 	    p++;
 	}
     }
-    return p;
+    return p-1;
 }
 
 /*
@@ -1014,8 +1036,10 @@ QuoteEnd(string, term)
  *	the end of the variable name spec.
  *
  * Results:
- *	The return value is a pointer to the character just after the
- *	last one that is part of the variable name.
+ *	The return value is a pointer to the last character that
+ *	is part of the variable name.  If the variable name doesn't
+ *	terminate properly then the return value is a pointer to the
+ *	null character at the end of the string.
  *
  * Side effects:
  *	None.
@@ -1030,18 +1054,18 @@ VarNameEnd(string)
     register char *p = string+1;
 
     if (*p == '{') {
-	do {
-	    p++;
-	} while ((*p != '}') && (*p != 0));
-    } else {
-	while (isalnum(*p) || (*p == '_')) {
-	    p++;
-	}
-	if ((*p == '(') && (p != string+1)) {
-	    p = QuoteEnd(p+1, ')');
+	for (p++; (*p != '}') && (*p != 0); p++) {
+	    /* Empty loop body. */
 	}
+	return p;
     }
-    return p;
+    while (isalnum(*p) || (*p == '_')) {
+	p++;
+    }
+    if ((*p == '(') && (p != string+1)) {
+	return QuoteEnd(p+1, ')');
+    }
+    return p-1;
 }
 
 /*
@@ -1161,6 +1185,9 @@ Tcl_ParseVar(interp, string, termPtr)
 	*termPtr = string;
     }
 
+    if (((Interp *) interp)->noEval) {
+	return "";
+    }
     c = *name1End;
     *name1End = 0;
     result = Tcl_GetVar2(interp, name1, name2, TCL_LEAVE_ERR_MSG);
