@@ -20,7 +20,7 @@
  * software for any purpose.  It is provided "as is" without
  * express or implied warranty.
  *
- * $Header: /user6/ouster/tcl/RCS/tclUnix.h,v 1.26 92/08/03 08:27:43 ouster Exp $ SPRITE (Berkeley)
+ * $Header: /user6/ouster/tcl/RCS/tclUnix.h,v 1.28 92/08/21 15:59:14 ouster Exp $ SPRITE (Berkeley)
  */
 
 #ifndef _TCLUNIX
@@ -290,9 +290,13 @@ extern int	access	   _ANSI_ARGS_((CONST char *path, int mode));
 extern int	chdir	   _ANSI_ARGS_((CONST char *path));
 extern int	close	   _ANSI_ARGS_((int fd));
 extern int	dup2	   _ANSI_ARGS_((int src, int dst));
+extern void	endpwent   _ANSI_ARGS_((void));
 extern int	execvp	   _ANSI_ARGS_((CONST char *name, char **argv));
 extern void	_exit 	   _ANSI_ARGS_((int status));
 extern pid_t	fork	   _ANSI_ARGS_((void));
+#ifndef MIPSEL
+extern long	fseek	   _ANSI_ARGS_((FILE *stream, long offset, int base));
+#endif
 extern uid_t	geteuid	   _ANSI_ARGS_((void));
 extern pid_t	getpid	   _ANSI_ARGS_((void));
 extern char *	getcwd 	   _ANSI_ARGS_((char *buffer, int size));
@@ -300,7 +304,7 @@ extern char *	getwd  	   _ANSI_ARGS_((char *buffer));
 extern int	kill	   _ANSI_ARGS_((pid_t pid, int sig));
 extern long	lseek	   _ANSI_ARGS_((int fd, int offset, int whence));
 extern char *	mktemp	   _ANSI_ARGS_((char *template));
-#ifndef ultrix
+#if !(defined(sparc) || defined(_IBMR2) || defined(MIPSEL))
 extern int	open	   _ANSI_ARGS_((CONST char *path, int flags, ...));
 #endif
 extern int	pipe	   _ANSI_ARGS_((int *fdPtr));
