@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static char rcsid[] = "$Header: /user6/ouster/tcl/RCS/tclVar.c,v 1.27 92/05/07 09:24:59 ouster Exp $ SPRITE (Berkeley)";
+static char rcsid[] = "$Header: /user6/ouster/tcl/RCS/tclVar.c,v 1.28 92/09/23 09:10:31 ouster Exp $ SPRITE (Berkeley)";
 #endif
 
 #include "tclInt.h"
@@ -449,6 +449,8 @@ Tcl_SetVar2(interp, name1, name2, newValue, flags)
 	newVarPtr->valueLength = varPtr->valueLength;
 	newVarPtr->upvarUses = varPtr->upvarUses;
 	newVarPtr->tracePtr = varPtr->tracePtr;
+	newVarPtr->searchPtr = varPtr->searchPtr;
+	newVarPtr->flags = varPtr->flags;
 	strcpy(newVarPtr->value.string, varPtr->value.string);
 	Tcl_SetHashValue(hPtr, newVarPtr);
 	ckfree((char *) varPtr);
