@@ -1,4 +1,4 @@
-/* Copyright 1990,91 GROUPE BULL -- See license conditions in file COPYRIGHT */
+/* Copyright 1990-92 GROUPE BULL -- See license conditions in file COPYRIGHT */
 /*****************************************************************************\
 * xpmP.h:                                                                     *
 *                                                                             *
@@ -41,7 +41,7 @@ extern FILE *popen();
 #endif
 #endif
 
-#ifdef SYSV
+#if defined(SYSV) || defined(SVR4)
 #define bcopy(source, dest, count) memcpy(dest, source, count)
 #endif
 
@@ -128,6 +128,12 @@ typedef struct {
 
 /* XPM private routines */
 
+FUNC(xpmWriteData, int, (xpmData * mdata,
+		    xpmInternAttrib * attrib, XpmAttributes * attributes));
+
+FUNC(xpmCreateData, int, (char ***data_return,
+		    xpmInternAttrib * attrib, XpmAttributes * attributes));
+
 FUNC(xpmParseDataAndCreateImage, int, (xpmData * data,
 				       Display * display,
 				       XImage ** image_return,
@@ -175,7 +181,7 @@ FUNC(xpmNextWord, unsigned int, (xpmData * mdata, char *buf));
 FUNC(xpmGetCmt, int, (xpmData * mdata, char **cmt));
 FUNC(xpmReadFile, int, (char *filename, xpmData * mdata));
 FUNC(xpmWriteFile, int, (char *filename, xpmData * mdata));
-FUNC(xpmOpenArray, int, (char **data, xpmData * mdata));
+FUNC(xpmOpenArray, void, (char **data, xpmData * mdata));
 FUNC(XpmDataClose, int, (xpmData * mdata));
 
 /* RGB utility */
@@ -185,10 +191,10 @@ FUNC(xpmGetRgbName, char *, (xpmRgbName * rgbn, int rgbn_max,
 			     int red, int green, int blue));
 FUNC(xpmFreeRgbNames, void, (xpmRgbName * rgbn, int rgbn_max));
 
-FUNC(xpm_xynormalizeimagebits, int, (register unsigned char *bp,
+FUNC(xpm_xynormalizeimagebits, void, (register unsigned char *bp,
+				      register XImage * img));
+FUNC(xpm_znormalizeimagebits, void, (register unsigned char *bp,
 				     register XImage * img));
-FUNC(xpm_znormalizeimagebits, int, (register unsigned char *bp,
-				    register XImage * img));
 
 /*
  * Macros
@@ -256,12 +262,16 @@ typedef struct {
 } xpmHashTable;
 
 FUNC(xpmHashTableInit, int, (xpmHashTable *table));
-FUNC(xpmHashTableFree, int, (xpmHashTable *table));
+FUNC(xpmHashTableFree, void, (xpmHashTable *table));
 FUNC(xpmHashSlot, xpmHashAtom *, (xpmHashTable *table, char *s));
-FUNC(xpmHashIntern, xpmHashAtom, (xpmHashTable *table, char *tag, void *data));
+FUNC(xpmHashIntern, int, (xpmHashTable *table, char *tag, void *data));
 
 #define HashAtomData(i) ((void *)i)
 #define HashColorIndex(slot) ((unsigned int)((*slot)->data))
-#define USE_HASHTABLE (ncolors > 4)
+#define USE_HASHTABLE (cpp > 2 && ncolors > 4)
+
+#ifdef NEED_STRDUP
+FUNC(strdup, char *, (char *s1));
+#endif
 
 #endif
